#include <math.h>
#include <string.h>
#include "share.h"
#include "menu.h"

uchar tile_attr[256];
Map map;


THE_COLOURS

int OX,OY,n_maps;

//FILE *fr;
DATAFILE *D;
BITMAP *buf,*copy_buf;

V3D flag_v[17][9];

void draw_tile(int tx,int ty)
{
  BITMAP *b;
  if (tx>=0 && tx<map.w && ty>=0 && ty<map.h) {
    b=BT(map.b[tx][ty]);
    draw_sprite(buf,b,tx*TILE_SIZE-OX,ty*TILE_SIZE-OY);
  }
}

void draw_tiles()
{
  int xfrom,xto,yfrom,yto,tx,ty;

  xfrom=OX/TILE_SIZE-1; xto=(OX+buf->w)/TILE_SIZE;
  yfrom=OY/TILE_SIZE-1; yto=(OY+buf->h)/TILE_SIZE;

  for (ty=yfrom; ty<=yto; ty++) {
    for (tx=xfrom; tx<=xto; tx++)
      draw_tile(tx,ty);
  }
}


void tiles_init()
{
  int x,y;
  FILE *f;
  
  f=fopen("tileattr.qk","rb");
  fread(tile_attr,1,256,f);
  fclose(f);

  map.w=map.h=25;
  map.n_ins=0;
  for (y=0; y<25; y++)
    for (x=0; x<25; x++)
      map.a[x][y]=(x%24==0)||(y%24==0) ? 1 : 2;
}

bool tile_isnt(int x,int y,int n)
{
  if (x<0 || y<0 || x>=map.w || y>=map.h) return true;
  return map.a[x][y] != n;
}

int mapa(int x,int y)
{
  if (x<0 || y<0 || x>=map.w || y>=map.h) return 127;
  return map.a[x][y];
}

void calc_tile(int x,int y)
{
  int a,b,c;
  b=a=map.a[x][y];
  switch(a) {
    case 38:
      map.b[x][y]=0x57
         + (tile_isnt(x,y+1,38) ? 16:0)
         - (tile_isnt(x,y-1,38) ? 16:0)
         + (tile_isnt(x+1,y,38) ? 1:0)
         - (tile_isnt(x-1,y,38) ? 1:0);

    Case 1: c=mapa(x,y+1);
            if (c==0x39 || c==0x59 || c==0x79) b=c-16;
            else b=0;
    Case 0x39: case 0x59: case 0x79:
    if ((x+y)&1) b++;
  }
  map.b[x][y]=b;
}

void calc_tile_all()
{
  for (int x=0; x<map.w; x++)
    for (int y=0; y<map.h; y++)
      calc_tile(x,y);
}


void insert_row(int ty)
{
  int x,y;
  for (x=0;x<map.w;x++)
    for(y=map.h;y>ty;y--)
      map.a[x][y]=map.a[x][y-1];
  map.h++;
  for (x=0;x<map.n_ins;x++)
    if (map.i[x].y>TILE_SIZE*ty)
      map.i[x].y+=TILE_SIZE;
}

void delete_row(int ty)
{
  int x,y;
  for (x=0;x<map.w;x++)
    for(y=ty;y<map.h;y++)
      map.a[x][y]=map.a[x][y+1];
  map.h--;
  for (x=0;x<map.n_ins;x++)
    if (map.i[x].y>TILE_SIZE*ty)
      map.i[x].y-=TILE_SIZE;
}

void insert_column(int tx)
{
  int x,y;
  for (y=0;y<map.h;y++)
    for(x=map.w;x>tx;x--) 
      map.a[x][y]=map.a[x-1][y];
  map.w++;
  for (y=0;y<map.n_ins;y++)
    if (map.i[y].x>TILE_SIZE*tx)
      map.i[y].x+=TILE_SIZE;
}

void delete_column(int tx)
{
  int x,y;
  for (y=0;y<map.h;y++)
    for(x=tx;x<map.w;x++)
      map.a[x][y]=map.a[x+1][y];
  map.w--;
  for (y=0;y<map.n_ins;y++)
    if (map.i[y].x>TILE_SIZE*tx)
      map.i[y].x-=TILE_SIZE;
}

void load_map(char *fn)
{
  int x,y;
  FILE *f;

  f=fopen(fn,"rb");
  if (!f) { /*fprintf(fr,"Couldn't open file \"%s\"",fn);*/ exit(0); }

  map.w=    fmgetw(f);
  map.h=    fmgetw(f);
  map.n_ins=fmgetw(f);

  for (x=0; x<map.w; x++)
    for (y=0; y<map.h; y++)
      map.a[x][y] = fgetc(f);

  for (x=0; x<map.n_ins; x++) {
    map.i[x].x=fmgetl(f);
    map.i[x].y=fmgetl(f);
    map.i[x].n=fmgetl(f);
  }
  calc_tile_all();

  fclose(f);
}

void save_map(char *fn)
{
  int x,y;
  FILE *f;
  f=fopen(fn,"wb");

  fmputw(map.w,f);
  fmputw(map.h,f);
  fmputw(map.n_ins,f);

  for (x=0; x<map.w; x++)
    for (y=0; y<map.h; y++)
      fputc(map.a[x][y],f);

  for (x=0; x<map.n_ins; x++) {
    fmputl(map.i[x].x,f);
    fmputl(map.i[x].y,f);
    fmputl(map.i[x].n,f);
  }
  fclose(f);

  f=fopen("tileattr.qk","wb");
  fwrite(tile_attr,1,256,f);
  fclose(f);
}

int count_maps()
{
  char s[15];
  n_maps=0;
  for (int j=1;j<100;j++) {
    sprintf(s,"m%02d.map",j);
    if (exists(s)) n_maps=j;
    else break;
  }
  return n_maps;
}

int vm=GFX_AUTODETECT;
void share_init()
{
//  fr=fopen("report.txt","w");

  srand(time(NULL));

  allegro_init();
  install_timer();
  install_keyboard();
//  printf("Initialising joystick...");
#ifdef JOY_TYPE_4BUTTON
  if (!stricmp(getenv("WHICH_MACHINE"),"Love_PC2"))
    install_joystick(JOY_TYPE_4BUTTON);
  else
#endif
    install_joystick(JOY_TYPE_AUTODETECT);
  //  printf(" done.\n");

  reserve_voices(4,4);
  if (install_sound(DIGI_AUTODETECT,MIDI_AUTODETECT,NULL)) {
    allegro_message(allegro_error);
  }

  set_color_depth(24);
  if (set_gfx_mode(vm,640,480,0,0)) {
    set_color_depth(16);
    if (set_gfx_mode(vm,640,480,0,0)) {
      set_color_depth(15);
      if (set_gfx_mode(vm,640,480,0,0)) {
        allegro_message(allegro_error);
        exit(1);
      }
    }
  }

  buf=create_bitmap(SCREEN_W,SCREEN_H);
  copy_buf=create_bitmap(256,128);
  D=load_datafile("data.dat");

//  play_midi((MIDI*)D[MUSIC].dat,1);
  midi_seek(rand()%880+2);

  tiles_init();
  count_maps();

  col_red=makecol(255,0,0);
  col_white=makecol(255,255,255);
  col_black=makecol(0,0,0);
  col_green=makecol(0,255,0);
  col_blue=makecol(0,0,255);
  col_cyan=makecol(0,255,255);
  col_magenta=makecol(255,0,255);
  col_yellow=makecol(255,255,0);
  col_grey=makecol(128,128,128);
  col_gold=makecol(128,64,0);



  LOCK_VARIABLE(t1);
  LOCK_FUNCTION(timer_click);
  install_int_ex(timer_click,BPS_TO_TIMER(timer_speed));
 
}

void share_cleanup()
{
//  fclose(fr);
  destroy_bitmap(buf);
  destroy_bitmap(copy_buf);
  unload_datafile(D);
}

int main(int argc,char *argv[])
{
  int j,x,y;
  bool run_maped=false;
  char *p,*fn=NULL;
 
  for (j=1; j<argc; j++) {
    if (!strcmp(argv[j],"-m")) run_maped=true;
    if (!strcmp(argv[j],"-w")) vm=GFX_AUTODETECT_WINDOWED;
    if (!strcmp(argv[j],"-f")) vm=GFX_AUTODETECT_FULLSCREEN;

    if ((p=strrchr(argv[j],'.')))
      if (!stricmp(p,".map"))
        fn=argv[j];
  }
  
  share_init();

  BITMAP *menu_buf=create_bitmap(500,200);
  menu_font=(FONT*)D[FONT1].dat;

  MenuList ml,ml1;

  MenuItem i11("Australia",0), i12("Lego",0), i13("TINS",0), i14("Recursive",0);
  ml1 << i11 << i12 << i13 << i14;
  MenuItem i1("Play!",1), i2("Editor",2), i3("Flags",&ml1), i5("Quit",-1);
  ml << i1 << i2 << i3 << i5;
  ml1.index=rand()%3;
  MenuSet ms(&ml);
  ms.linecol   =col_black;
  ms.wordcol0  =col_black;
  ms.wordcol1  =col_black;
  ms.wordcol2  =col_black;
  ms.hilightcol=col_yellow;

  int k,r;
  double th;

  t1=1; t2=0;
  do {
    if (t1>t2) {
      r=0;
      if (keypressed()) {
        k=readkey();
        r=ms.OnKey(k);

        if (r==1) play_game();
        if (r==2) maped_main(NULL);
      } else {
        for (x=0;x<17;x++) {
          for (y=0;y<9;y++) {
            th=t2/12.0+x/5.0;
            V3D &v1=flag_v[x][y];
            v1.x=ftofix(270+20*x);
            v1.y=ftofix(240.0+20*y-x*1.5*sin(th));
            v1.u=itofix(x*10);
            v1.v=itofix(y*10);
            v1.c=(int)(205+50*cos(th)-2.2*y);
          }
        }
      }
      t2++;
      if (r==1 || r==2) { t1=1; t2=0; stop_midi(); }
    } else {
      BITMAP *b1;
      if (ml1.index==3) b1=copy_buf; else b1=BD(F2+ml1.index);

      clear_to_color(buf,col_white);
      clear_to_color(menu_buf,col_white);
      ms.Draw(menu_buf);
      blit(menu_buf,buf,0,0,0,240,menu_buf->w,menu_buf->h);

      for (x=0;x<16;x++) {
        for (y=0;y<8;y++) {
          quad3d(buf,POLYTYPE_ATEX_LIT,b1,&flag_v[x][y],&flag_v[x+1][y],&flag_v[x+1][y+1],&flag_v[x][y+1]);
        }
      }

      blit((BITMAP*)D[HEADER].dat,buf,0,0,0,0,SCREEN_W,200);

      stretch_blit(buf,copy_buf,0,0,SCREEN_W,SCREEN_H,0,0,160,80); 
      blit(buf,screen,0,0,0,0,SCREEN_W,SCREEN_H); 
//    if (r==1 || r==2) while (keypressed()) readkey();
    }
  } while (r!=-1);

  destroy_bitmap(menu_buf);
  share_cleanup();
  return 0;
}
END_OF_MAIN();
